package com.parksong.basics.util.configurations;

import com.zaxxer.hikari.HikariDataSource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import javax.sql.DataSource;

/**
 * @author LiuJS
 * @Time 2021/7/15 14:38
 * @Desc
 * @Version 1.0
 */
@Slf4j
@Configuration
public class DataSourceConfig {
    @Value("${spring.datasource.url}")
    private String url;
    @Value("${spring.datasource.username}")
    private String username;
    @Value("${spring.datasource.password}")
    private String password;
    @Value("${spring.datasource.driver-class-name}")
    private String driverClassName;
    @Value("${spring.datasource.type}")
    private String type;
    @Value("${spring.datasource.max-idle}")
    private Integer maxIdle;
    @Value("${spring.datasource.min-idle}")
    private Integer minIdle;
    @Value("${spring.datasource.max-wait}")
    private Integer maxWait;
    @Value("${spring.datasource.initial-size}")
    private Integer initialSize;

    /**
     * 配置连接池信息
     * @return
     */
    @Bean
    public DataSource dataSource(){
        log.info("----------------------开始进行数据库连接------------------------");
        HikariDataSource dataSource = new HikariDataSource();
        //添加数据库数据
        dataSource.setJdbcUrl(url);
        dataSource.setUsername(username);
        dataSource.setPassword(password);
        dataSource.setDriverClassName(driverClassName);

        dataSource.setMinimumIdle(minIdle);
        try {
            dataSource.getConnection();
            log.info("-----------------------数据库连接成功---------------------------");
        }catch (Exception e){
            log.info("-----------------------数据库连接失败----------------------------");
            e.printStackTrace();
        }finally {
            return dataSource;
        }
    }
}
