package com.parksong.controllers.applets;

import com.parksong.basics.api.ApiVersion;
import com.parksong.basics.beans.PageResultNo;
import com.parksong.basics.beans.ResultNo;
import com.parksong.basics.filter.ApiFilter;
import com.parksong.beans.delivery.ResponsePage;
import com.parksong.beans.delivery.StatisticsOutboundRate;
import com.parksong.beans.express.*;
import com.parksong.filter.applets.AppletsFilter;
import com.parksong.services.applets.AppletsService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;

/**
 * @author LiuJS
 * @Time 2021/7/6 11:04
 * @Desc 小程序接口
 * @Version 1.0
 */
@RestController
@ResponseBody
@RequestMapping("applets")
public class AppletsController {

    @Autowired
    private AppletsService appletsService;



    /*-----------------------------------------------------用户模块-----------------------------------------------------*/

    /**
     * 用户登录
     * @param filter
     * @return
     * @throws Exception
     */
    @ApiVersion(1)
    @PostMapping("{version}/user/login")
    public ResultNo<String> userLogin(@RequestBody ApiFilter<AppletsFilter> filter) throws Exception {
        return new ResultNo<>(appletsService.userLogin(filter.getAccount(), filter.getPassword()));
    }

    /**
     * 用户详情信息
     * @param filter
     * @return
     */
    @ApiVersion(1)
    @PostMapping("{version}/user/detail")
    public ResultNo<ExpressDept> userDetail(@RequestBody ApiFilter<AppletsFilter> filter){
        return new ResultNo<>(appletsService.userDetail(filter.getData()));
    }

    //修改用户信息

    //修改密码


    /*-----------------------------------------------------业务统计模块-----------------------------------------------------*/

    /**
     * 查询快递信息接口
     * @param filter
     * @return
     */
    @ApiVersion(1)
    @PostMapping("{version}/express/info")
    public PageResultNo<List<ExpressApplets>> expressInfo(@RequestBody ApiFilter<AppletsFilter> filter){
        ResponsePage responsePage = appletsService.expressInfo(filter.getData());
        return new PageResultNo<>(responsePage.getExpressAppletsList(), responsePage.getTotal());
    }

    /**
     * 业务统计
     * @param filter
     * @return
     */
    @ApiVersion(1)
    @PostMapping("{version}/express/statistics")
    public PageResultNo<StatisticsOutboundRate> expressStatistics(@RequestBody ApiFilter<AppletsFilter> filter){
        return new PageResultNo<>(appletsService.expressStatistics(filter.getData()));
    }

    /*-----------------------------------------------------消费统计模块-----------------------------------------------------*/

    /**
     * 消费统计
     * @param filter
     * @return
     */
    @ApiVersion(1)
    @PostMapping("{version}/consume/statistics")
    public ResultNo<List<ConsumeStatistics>> consumeStatistics(@RequestBody ApiFilter<AppletsFilter> filter){
        return new ResultNo<>(appletsService.consumeStatistics(filter.getData()));
    }

    /**
     * 消费详情统计
     * @param filter
     * @return
     */
    @ApiVersion(1)
    @PostMapping("{version}/consume/statistics/detail")
    public ResultNo<List<ConsumeDetail>> consumeDetail(@RequestBody ApiFilter<AppletsFilter> filter){
        return new ResultNo<>(appletsService.consumeDetail(filter.getData()));
    }

    /**
     * 充值统计
     * @param filter
     * @return
     */
    @ApiVersion(1)
    @PostMapping("{version}/recharge/statistics")
    public ResultNo<List<RechargeStatistics>> rechargeStatistics(@RequestBody ApiFilter<AppletsFilter> filter){
        return new ResultNo<>(appletsService.rechargeStatistics(filter.getData()));
    }

    /*-----------------------------------------------------充值模块-----------------------------------------------------*/

    /**
     * 消充值模块
     * @param filter
     * @return
     */
    @ApiVersion(1)
    @PostMapping("{version}/recharge/order_mes")
    public ResultNo<Map<String, Object>> recharge(@RequestBody ApiFilter<AppletsFilter> filter){
        return new ResultNo<>(appletsService.recharge(filter));
    }
    //微信充值
    //回调地址 需要外网能够访问
    @ApiVersion(1)
    @PostMapping("{version}/recharge/notify_url")
    public String notifyUrl(HttpServletRequest request, HttpServletResponse response) throws Exception {
        appletsService.notifyUrl(request,response);
        return "";
    }

}
