package com.parksong.services.applets.impl;

import com.alibaba.fastjson.JSONObject;
import com.parksong.basics.exception.BusinessException;
import com.parksong.basics.filter.ApiFilter;
import com.parksong.basics.util.redis.CacheTool;
import com.parksong.beans.applets.OrderPayBean;
import com.parksong.beans.delivery.ResponsePage;
import com.parksong.beans.delivery.StatisticsOutboundRate;
import com.parksong.beans.express.*;
import com.parksong.dao.applets.AppletsMapper;
import com.parksong.filter.applets.AppletsFilter;
import com.parksong.filter.delivery.DeliveryFilter;
import com.parksong.services.applets.AppletsService;
import com.parksong.services.delivery.DeliverysService;
import com.parksong.util.Http.OkHttpUtil;
import com.parksong.util.JwtUtils;
import com.parksong.util.WxPayUtil;
import com.parksong.util.enums.ExpressStatusEnum;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.DigestUtils;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.*;

/**
 * @author LiuJS
 * @Time 2021/7/6 11:12
 * @Desc
 * @Version 1.0
 */
@Service
@Slf4j
public class AppletsServiceImpl implements AppletsService {

    @Autowired
    private AppletsMapper appletsMapper;

    @Autowired
    private DeliverysService deliverysService;

    @Resource
    private WxPayUtil wxPayUtil;
    //商户密钥
    @Value("${wechat.weChatKey}")
    private String weChatKey;

    //小程序appid
    @Value("${wechat.appId}")
    private String appId;

    //微信api地址
    @Value("${wechat.weChatApi}")
    private String weChatApi;

    //小程序密钥
    @Value("${wechat.mch_id}")
    private String mch_id;

    //小程序密钥
    @Value("${wechat.secret}")
    private String secret;

    public String userLogin(String account, String password) throws Exception {
        if (StringUtils.isBlank(account) || StringUtils.isBlank(password)){
            throw new BusinessException("账号或者密码不能为空");
        }
        ExpressDept expressDept = appletsMapper.userLogin(account);
        if (expressDept == null){
            throw new BusinessException("账号不存在或者账号状态出现异常");
        }else {
            if (DigestUtils.md5DigestAsHex(("youbaimi" + password).getBytes()).equals(expressDept.getPassword())){
                Map<String, String> claims = new HashMap<>(3);
                claims.put("uid", expressDept.getId().toString());
                claims.put("storeid", expressDept.getStoreId().toString());
                claims.put("expid", expressDept.getExpId().toString());
                claims.put("uname", expressDept.getOwnerName());
                claims.put("uphone", expressDept.getOwnerPhone());
                return JwtUtils.createToken(claims);
            } else {
                throw new BusinessException("密码错误");
            }
        }

    }

    @Override
    public ExpressDept userDetail(AppletsFilter filter) {
        return appletsMapper.userDetail(filter.getExpressDept().getId());
    }

    @Override
    public ResponsePage expressInfo(AppletsFilter filter) {
        ResponsePage responsePage = new ResponsePage();
        // 查询前的filter数据处理
        this.queryDeliveryFilterHandle(filter);
        List<ExpressApplets> expressApplets = appletsMapper.expressInfo(filter);
        expressApplets.stream().forEach(express -> express.setStatusName(ExpressStatusEnum.valueOf(express.getStatus()).getValue()));
        responsePage.setExpressAppletsList(expressApplets);
        responsePage.setTotal(appletsMapper.selectExpressInfoCount(filter));
        return responsePage;
    }

    @Override
    public StatisticsOutboundRate expressStatistics(AppletsFilter filter) {
        DeliveryFilter deliveryFilter = new DeliveryFilter();
        deliveryFilter.setStoreId(filter.getStoreId());
        deliveryFilter.setBeginTime(filter.getStartTime());
        deliveryFilter.setEndTime(filter.getEndTime());
        deliveryFilter.setExpId(filter.getExpressDept().getExpId().intValue());
        return deliverysService.outboundRate(deliveryFilter);
    }

    @Override
    public List<ConsumeStatistics> consumeStatistics(AppletsFilter filter) {
        filter.setExpressId(filter.getExpressDept().getExpId().intValue());
        List<ConsumeStatistics> consumeStatisticsList = appletsMapper.consumeStatistics(filter);
        consumeStatisticsList.stream().forEach(consumeStatistics -> consumeStatistics.setConsumeAmount(new BigDecimal(consumeStatistics.getNumber()).multiply(consumeStatistics.getPrice())));
        return consumeStatisticsList;
    }
    @Override
    public List<ConsumeDetail> consumeDetail(AppletsFilter filter){
        filter.setExpressId(filter.getExpressDept().getExpId().intValue());
        List<ConsumeDetail> consumeDetailList = appletsMapper.consumeDetail(filter);
        consumeDetailList.stream().forEach(consumeDetail -> consumeDetail.setAmount(new BigDecimal(consumeDetail.getPackageCount()).multiply(consumeDetail.getPackagePrice())));
        return consumeDetailList;
    }

    @Override
    public List<RechargeStatistics> rechargeStatistics(AppletsFilter filter){
        filter.setExpressId(filter.getExpressDept().getExpId().intValue());
        List<RechargeStatistics> rechargeStatisticsList = appletsMapper.rechargeStatistics(filter);
        return rechargeStatisticsList;
    }

    private void queryDeliveryFilterHandle(AppletsFilter filter){
        filter.sortInit();
    }

    @Override
    public Map<String, Object> recharge(ApiFilter<AppletsFilter> filter){

        String jsCode = filter.getData().getJsCode();
        BigDecimal amount = filter.getData().getAmount();
        //获取用户openid等相关参数
        String res = "";
        try {
            res = OkHttpUtil.get(weChatApi+"/sns/jscode2session?appid="+appId+"&secret="+secret+"&js_code="+jsCode+"&grant_type=authorization_code");
        } catch (Exception e) {
            e.printStackTrace();
        }
        JSONObject resJson = JSONObject.parseObject(res);
        String openId = resJson.getString("openid");
        ExpressDept user = appletsMapper.userDetail(filter.getData().getExpressDept().getId());
        //绑定\更新用户的openid
        user.setOpenId(openId);
        appletsMapper.updateDept(user);
        //获取客户端的ip地址
        //String spbill_create_ip = wxPayUtil.getIpAddr(request);
        String spbill_create_ip = filter.getIpAdress();
        //生成六位数随机字符串
        String nonce_str = wxPayUtil.createCode(6);
        String out_trade_no = wxPayUtil.createCode(30);

        //调用微信支付接口 生成订单
        //这是生成了一个订单类进行参数封装

        OrderPayBean orderPay = OrderPayBean.builder()
                //商品详情用于描述商品
                .body("金额充值")
                //随机生成的字符串不能超过32位
                .nonce_str(nonce_str)
                //商户订单号 自己订单表的id或者商户系统id 是自己系统的  不能超过32位
                .out_trade_no(out_trade_no)
                //ip地址 谁调用就是谁的地址 上面有用api获取
                .spbill_create_ip(spbill_create_ip)
                //金额 这里的金额实际业务需要乘 100 微信是按照分来算的 要转换位字符串 微信识别
                .total_fee(((amount.multiply(new BigDecimal(100))).setScale( 0, BigDecimal.ROUND_DOWN ).longValue()) + "")
                //用户的openId 这个需要调用微信获取openId 一般都是存储到用户表
                .openid(openId)
                .build();
        //返回下单接口
        Map<String, Object> objectMap = wxPayUtil.wxPay(orderPay);
        return objectMap;
    }

    public Map<String, Object> notifyUrl(HttpServletRequest request, HttpServletResponse response) throws Exception {

        log.info("--------------回调开始----------------");
        Enumeration e=request.getParameterNames();//获取所有参数名
        while(e.hasMoreElements()){//通过Enumeration类中的hasMoreElements()判断是否还有参数名
            String parameterName=(String)e.nextElement(); //获取当前参数名
            //再通过request.getParameter("")的方法来获取对应参数名的值
            System.out.println(parameterName+": "+request.getParameter(parameterName));
            log.info("parameterName:"+parameterName+": "+request.getParameter(parameterName));
        }

        //微信通过流传输
        BufferedReader br = new BufferedReader(new InputStreamReader(request.getInputStream()));
        String line = null;
        StringBuilder sb = new StringBuilder();
        while ((line = br.readLine()) != null) {
            sb.append(line);
        }
        br.close();
        //sb为微信返回的xml
        String notityXml = sb.toString();
        String resXml = "";
        Map map = wxPayUtil.doXMLParse(notityXml);
        String returnCode = (String) map.get("return_code");
        log.info("returnCode:"+returnCode);
        if ("SUCCESS".equals(returnCode)) {
            //验证签名是否正确
            Map<String, String> validParams = wxPayUtil.paraFilter(map);
            //回调验签时需要去除sign和空值参数
            String prestr = wxPayUtil.createLinkString(validParams);
            //生成签名
            String mysign = wxPayUtil.sign(prestr, weChatKey, "utf-8").toUpperCase();
            //根据微信官网的介绍，此处不仅对回调的参数进行验签，还需要对返回的金额与系统订单的金额进行比对等
            log.info("----------mysign:"+mysign+"--------------map.get(\"sign\"):"+map.get("sign"));
            if (mysign.equals(map.get("sign").toString())) {

                String openid = (String) map.get("openid");
                String cash_fee = (String) map.get("cash_fee");
                String day = String.format("%02d", LocalDate.now().getDayOfMonth());
                CacheTool rechargeCacheTool = CacheTool.initrechargeCacheTool();
                String number = String.format("%06d", rechargeCacheTool.incrBy(day, 1).intValue());
                BigDecimal amount = new BigDecimal(cash_fee).divide(new BigDecimal(100));

                //根据openid获取用户信息
                ExpressDept user = appletsMapper.userDetailByOpenId(openid);
                //添加充值记录
                RechargeStatistics rechargeStatistics = new RechargeStatistics();
                rechargeStatistics.setAmount(amount);
                rechargeStatistics.setPayWay("2");
                rechargeStatistics.setOutTradeNo(day+number);
                rechargeStatistics.setPayStatus("1");
                rechargeStatistics.setAccountId(user.getId().toString());
                rechargeStatistics.setOperator("admin");
                int  a = appletsMapper.insertSelective(rechargeStatistics);

                //更新余额
                ExpressDeptAccount userAccount = new ExpressDeptAccount();
                userAccount.setId(user.getId());
                userAccount.setBalance(amount);
                int b =appletsMapper.updateDeptAccount(userAccount);

                log.info("----------a:"+a+"--------------b:"+b);

                //返回通知微信已经接收到消息
                resXml = "<xml>" + "<return_code><![CDATA[SUCCESS]]></return_code>"
                        + "<return_msg><![CDATA[OK]]></return_msg>" + "</xml> ";

            }
        } else {
            resXml = "<xml>" + "<return_code><![CDATA[FAIL]]></return_code>"
                    + "<return_msg><![CDATA[报文为空]]></return_msg>" + "</xml> ";
        }

        BufferedOutputStream out = new BufferedOutputStream(
                response.getOutputStream());
        out.write(resXml.getBytes());
        out.flush();
        out.close();
        return null;
    }
}
