package com.parksong.util;

import com.parksong.beans.applets.OrderPayBean;
import com.parksong.services.applets.impl.AppletsServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.digest.DigestUtils;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletRequest;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.*;

/**
 * Function: 支付工具类 <br/>
 * date: 2021-07-14 <br/>
 *
 * @author weiyf
 * @version 1.0
 * @since JDK1.8
 * @see
 */
@Component
@Slf4j
public class WxPayUtil {

    //小程序appid
    @Value("${wechat.appId}")
    private String appId;
    //小程序密钥
    @Value("${wechat.mch_id}")
    private String mch_id;
    //商家shopKey
    @Value("${wechat.shopKey}")
    private String shopKey;

    @Value("${wechat.weChatKey}")
    private String weChatKey;
    //微信调用地址
    private String pay_url="https://api.mch.weixin.qq.com/pay/unifiedorder";
    //交易类型  不同业务需求选择不同  详见微信开发文档
    public String trade_type="JSAPI";
    //支付成功通知回调地址 需要外网能访问你的项目  可以使用花生壳进行内网穿透
    private String notify_url="https://ubmtest.uxskj.com/applets/v1/recharge/notify_url";

    //调用支付 统一下单 接口，返回Map数据
    public Map<String, Object> wxPay(OrderPayBean orderPay) {
        //controller层封装的对象传递过来
        orderPay.setAppid(appId);
        orderPay.setMch_id(mch_id);
        orderPay.setTrade_type(trade_type);
        orderPay.setKey(weChatKey);
        orderPay.setPay_url(pay_url);
        orderPay.setNotify_url(notify_url);
        try {
            //参数封装成一个map
            Map<String, String> packageParams = new HashMap<String, String>();
            packageParams.put("appid", orderPay.getAppid());
            packageParams.put("mch_id", orderPay.getMch_id());
            packageParams.put("nonce_str", orderPay.getNonce_str());
            packageParams.put("body", orderPay.getBody());
            packageParams.put("out_trade_no", orderPay.getOut_trade_no());
            packageParams.put("total_fee", orderPay.getTotal_fee());
            packageParams.put("spbill_create_ip", orderPay.getSpbill_create_ip());
            packageParams.put("notify_url", orderPay.getNotify_url());
            packageParams.put("trade_type", orderPay.getTrade_type());
            packageParams.put("openid", orderPay.getOpenid());

            // 除去数组中的空值和签名参数
            packageParams = WxPayUtil.paraFilter(packageParams);
            String prestr = WxPayUtil.createLinkString(packageParams); // 把数组所有元素，按照“参数=参数值”的模式用“&”字符拼接成字符串
            //MD5运算生成签名，这里是第一次签名，用于调用统一下单接口
            String mysign = WxPayUtil.sign(prestr, orderPay.getKey(), "utf-8").toUpperCase();
            //拼接统一下单接口使用的xml数据，要将上一步生成的签名一起拼接进去
            String xml = "<xml version='1.0' encoding='utf-8'>" + "<appid>" + orderPay.getAppid() + "</appid>"
                    + "<body><![CDATA[" + orderPay.getBody() + "]]></body>"
                    + "<mch_id>" + orderPay.getMch_id() + "</mch_id>"
                    + "<nonce_str>" + orderPay.getNonce_str() + "</nonce_str>"
                    + "<notify_url>" + orderPay.getNotify_url() + "</notify_url>"
                    + "<openid>" + orderPay.getOpenid() + "</openid>"
                    + "<out_trade_no>" + orderPay.getOut_trade_no() + "</out_trade_no>"
                    + "<spbill_create_ip>" + orderPay.getSpbill_create_ip() + "</spbill_create_ip>"
                    + "<total_fee>" + orderPay.getTotal_fee() + "</total_fee>"
                    + "<trade_type>" + orderPay.getTrade_type() + "</trade_type>"
                    + "<sign>" + mysign + "</sign>"
                    + "</xml>";
            System.out.println(xml);
            //调用统一下单接口，并接受返回的结果
            String result = WxPayUtil.httpRequest(orderPay.getPay_url(), "POST", xml);
            // 将解析结果存储在HashMap中
            Map map = WxPayUtil.doXMLParse(result);
            //返回状态码
            String return_code = (String) map.get("return_code");
            //返回给移动端需要的参数
            Map<String, Object> response = new HashMap<String, Object>();
            if (return_code == "SUCCESS" || return_code.equals(return_code)) {
                // 业务结果
                String prepay_id = (String) map.get("prepay_id");//返回的预付单信息
                response.put("nonceStr", orderPay.getNonce_str());
                response.put("package", "prepay_id=" + prepay_id);
                Long timeStamp = System.currentTimeMillis() / 1000;
                response.put("timeStamp", timeStamp + "");
                //这边要将返回的时间戳转化成字符串，不然小程序端调用wx.requestPayment方法会报签名错误
                String stringSignTemp = "appId=" + orderPay.getAppid() + "&nonceStr=" + orderPay.getNonce_str() + "&package=prepay_id=" + prepay_id + "&signType=" + "MD5" + "&timeStamp=" + timeStamp;
                //再次签名，这个签名用于小程序端调用wx.requesetPayment方法
                String paySign = WxPayUtil.sign(stringSignTemp, orderPay.getKey(), "utf-8").toUpperCase();
                System.out.println("再次签名paySign=" + paySign);
                response.put("paySign", paySign);
            }
            //返回处理结果 下单到此就处理完成了
            response.put("appid", orderPay.getOpenid());
            return response;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    //生成md5格式签名
    public static String sign(String text, String key, String input_charset) {
        text = text + "&key=" + key;
        return DigestUtils.md5Hex(getContentBytes(text, input_charset));
    }

    public static byte[] getContentBytes(String content, String charset) {
        if (charset == null || "".equals(charset)) {
            return content.getBytes();
        }
        try {
            return content.getBytes(charset);
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException("MD5签名过程中出现错误,指定的编码集不对,您目前指定的编码集是:" + charset);
        }
    }

    /**
     * 生成6位或10位随机数 param codeLength(多少位)
     *
     * @return
     */
    public static String createCode(int codeLength) {
        String code = "";
        for (int i = 0; i < codeLength; i++) {
            code += (int) (Math.random() * 9);
        }
        return code;
    }

    @SuppressWarnings("unused")
    private static boolean isValidChar(char ch) {
        if ((ch >= '0' && ch <= '9') || (ch >= 'A' && ch <= 'Z') || (ch >= 'a' && ch <= 'z'))
            return true;
        if ((ch >= 0x4e00 && ch <= 0x7fff) || (ch >= 0x8000 && ch <= 0x952f))
            return true;// 简体中文汉字编码
        return false;
    }

    /**
     * 除去数组中的空值和签名参数
     *
     * @param sArray 签名参数组
     * @return 去掉空值与签名参数后的新签名参数组
     */
    public static Map<String, String> paraFilter(Map<String, String> sArray) {
        Map<String, String> result = new HashMap<String, String>();
        if (sArray == null || sArray.size() <= 0) {
            return result;
        }
        for (String key : sArray.keySet()) {
            String value = sArray.get(key);
            if (value == null || value.equals("") || key.equalsIgnoreCase("sign")
                    || key.equalsIgnoreCase("sign_type")) {
                continue;
            }
            result.put(key, value);
        }
        return result;
    }

    /**
     * 把数组所有元素排序，并按照“参数=参数值”的模式用“&”字符拼接成字符串
     *
     * @param params 需要排序并参与字符拼接的参数组
     * @return 拼接后字符串
     */
    public static String createLinkString(Map<String, String> params) {
        List<String> keys = new ArrayList<String>(params.keySet());
        Collections.sort(keys);
        String prestr = "";
        for (int i = 0; i < keys.size(); i++) {
            String key = keys.get(i);
            String value = params.get(key);
            if (i == keys.size() - 1) {// 拼接时，不包括最后一个&字符
                prestr = prestr + key + "=" + value;
            } else {
                prestr = prestr + key + "=" + value + "&";
            }
        }
        return prestr;
    }

    public static String httpRequest(String requestUrl, String requestMethod, String outputStr) {
        // 创建SSLContext
        StringBuffer buffer = null;
        try {
            URL url = new URL(requestUrl);
            HttpURLConnection conn = (HttpURLConnection) url.openConnection();
            conn.setRequestMethod(requestMethod);
            conn.setDoOutput(true);
            conn.setDoInput(true);
            conn.connect();
            //往服务器端写内容
            if (null != outputStr) {
                OutputStream os = conn.getOutputStream();
                os.write(outputStr.getBytes("utf-8"));
                os.close();
            }
            // 读取服务器端返回的内容
            InputStream is = conn.getInputStream();
            InputStreamReader isr = new InputStreamReader(is, "utf-8");
            BufferedReader br = new BufferedReader(isr);
            buffer = new StringBuffer();
            String line = null;
            while ((line = br.readLine()) != null) {
                buffer.append(line);
            }
            br.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return buffer.toString();
    }

    public static Map doXMLParse(String strxml) throws Exception {
        if (null == strxml || "".equals(strxml)) {
            return null;
        }
        Map m = new HashMap();
        InputStream in = String2Inputstream(strxml);
        SAXBuilder builder = new SAXBuilder();
        Document doc = builder.build(in);
        Element root = doc.getRootElement();
        List list = root.getChildren();
        Iterator it = list.iterator();
        while (it.hasNext()) {
            Element e = (Element) it.next();
            String k = e.getName();
            String v = "";
            List children = e.getChildren();
            if (children.isEmpty()) {
                v = e.getTextNormalize();
            } else {
                v = getChildrenText(children);
            }
            m.put(k, v);
        }
//关闭流
        in.close();
        return m;
    }

    /**
     * 获取子结点的xml
     *
     * @param children
     * @return String
     */
    public static String getChildrenText(List children) {
        StringBuffer sb = new StringBuffer();
        if (!children.isEmpty()) {
            Iterator it = children.iterator();
            while (it.hasNext()) {
                Element e = (Element) it.next();
                String name = e.getName();
                String value = e.getTextNormalize();
                List list = e.getChildren();
                sb.append("<" + name + ">");
                if (!list.isEmpty()) {
                    sb.append(getChildrenText(list));
                }
                sb.append(value);
                sb.append("</" + name + ">");
            }
        }
        return sb.toString();
    }

    public static InputStream String2Inputstream(String str) throws UnsupportedEncodingException {
        return new ByteArrayInputStream(str.getBytes("utf-8"));
    }
    //获取IP
    public static String getIpAddr(HttpServletRequest request) {
        String ip = request.getHeader("X-Forwarded-For");
        if (org.apache.commons.lang3.StringUtils.isNotEmpty(ip) && !"unKnown".equalsIgnoreCase(ip)) {
            //多次反向代理后会有多个ip值，第一个ip才是真实ip
            int index = ip.indexOf(",");
            if (index != -1) {
                return ip.substring(0, index);
            } else {
                return ip;
            }
        }
        ip = request.getHeader("X-Real-IP");
        if (org.apache.commons.lang3.StringUtils.isNotEmpty(ip) && !"unKnown".equalsIgnoreCase(ip)) {
            return ip;
        }
        return request.getRemoteAddr();

    }

}
